/*
 * Copyright [2013-2021], Alibaba Group Holding Limited
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.alibaba.polardbx.executor.ddl.job.builder;

import com.alibaba.polardbx.optimizer.context.ExecutionContext;
import com.alibaba.polardbx.optimizer.core.planner.SqlConverter;
import com.alibaba.polardbx.optimizer.core.rel.ReplaceTableNameWithQuestionMarkVisitor;
import com.alibaba.polardbx.optimizer.core.rel.ddl.LogicalDropTable;
import com.alibaba.polardbx.optimizer.core.rel.ddl.data.DropTablePreparedData;
import org.apache.calcite.plan.RelOptCluster;
import org.apache.calcite.rel.core.DDL;
import org.apache.calcite.rel.ddl.DropTable;
import org.apache.calcite.sql.SqlDdlNodes;
import org.apache.calcite.sql.SqlDropTable;
import org.apache.calcite.sql.SqlIdentifier;
import org.apache.calcite.sql.parser.SqlParserPos;

import java.util.List;
import java.util.Map;

public class DropPhyTableBuilder extends DdlPhyPlanBuilder {

    private final DropTablePreparedData preparedData;

    public DropPhyTableBuilder(DDL ddl, DropTablePreparedData preparedData,
                               Map<String, List<List<String>>> tableTopology, ExecutionContext executionContext) {
        super(ddl, preparedData, executionContext);
        this.preparedData = preparedData;
        this.tableTopology = tableTopology;
    }

    public static DropPhyTableBuilder createBuilder(String schemaName,
                                                    String logicalTableName,
                                                    boolean ifExists,
                                                    Map<String, List<List<String>>> tableTopology,
                                                    ExecutionContext executionContext) {
        ReplaceTableNameWithQuestionMarkVisitor visitor =
            new ReplaceTableNameWithQuestionMarkVisitor(schemaName, executionContext);

        SqlIdentifier logicalTableNameNode = new SqlIdentifier(logicalTableName, SqlParserPos.ZERO);

        SqlDropTable sqlDropTable = SqlDdlNodes.dropTable(SqlParserPos.ZERO, ifExists, logicalTableNameNode, true);
        sqlDropTable = (SqlDropTable) sqlDropTable.accept(visitor);

        final RelOptCluster cluster = SqlConverter.getInstance(executionContext).createRelOptCluster(null);
        DropTable dropTable = DropTable.create(cluster, sqlDropTable, logicalTableNameNode);

        LogicalDropTable logicalDropTable = LogicalDropTable.create(dropTable);
        logicalDropTable.setSchemaName(schemaName);
        logicalDropTable.prepareData();

        return new DropPhyTableBuilder(dropTable, logicalDropTable.getDropTablePreparedData(), tableTopology,
            executionContext);
    }

    @Override
    protected void buildTableRuleAndTopology() {
    }

    @Override
    public void buildPhysicalPlans() {
        buildSqlTemplate();
        buildPhysicalPlans(preparedData.getTableName());
    }

}
